package de.XPortLib.Interfaces;

import java.util.Calendar;

import de.XPortLib.TimerEvent;

public interface XPortControllerAsyncIF {

	/**
	 * default getter method 
	 * @return 
	 * 		the current set IP Address for the XPort
	 */
	public abstract String getIpAddress();

	/**
	 * sets the IP Address for the XPort 
	 */
	public abstract void setIpAddress(String ipAddres);

	/**
	 * default getter method 
	 * @return 
	 * 		the current set port for the XPort
	 */
	public abstract int getPort();

	/**
	 * sets the port for the XPort 
	 */
	public abstract void setPort(int port);

	/**
	 * reads the current relais status
	 * @return
	 * 			boolean array with status for each relais
	 */
	public abstract void readRelaisAsync(final XPortCallbackIF callback);

	/**
	 * toggles the relais with the given number.
	 * @param channel
	 * 			relais number to toggle
	 * @param callback
	 * 			the callback object to be called after finishing the task
	 * @return
	 * 			new relais status after switching
	 */
	public abstract void toggleRelaisAsync(final int channel,
			final XPortCallbackIF callback);

	/**
	 * reads the timerevent at given position
	 * @param nr
	 * 			the position
	 * @param callback
	 * 			the callback object to be called after finishing the task
	 * @return
	 * 			the TimerEvent object retrieved from XPort
	 */
	public abstract void readTimerAsync(final int nr,
			final XPortCallbackIF callback);

	/**
	 * reads all timerevents
	 * @param callback
	 * 			the callback object to be called after finishing the task
	 * @return
	 * 			the array of TimerEvents
	 */
	public abstract void readTimersAsync(final XPortCallbackIF callback);

	/**
	 * writes a timerevent to the given position (existing events are overwritten)
	 * @param nr
	 * 			the position
	 * @param timer
	 * 			the timer object to be saved
	 * @param callback
	 * 			the callback object to be called after finishing the task
	 * @return
	 * 			the total number of events after writing this new event
	 */
	public abstract void writeTimerAsync(final int nr, final TimerEvent timer,
			final XPortCallbackIF callback);

	/**
	 * writes a timerevent to the given position (existing events are overwritten)
	 * @param nr
	 * 			the position
	 * @param callback
	 * 			the callback object to be called after finishing the task
	 * @return
	 * 			the total number of events after writing this new event
	 */
	public abstract void deleteTimerAsync(final int nr,
			final XPortCallbackIF callback);

	/**
	 * sets Date and Time
	 * @param callback
	 * 			the callback object to be called after finishing the task
	 * @param cal
	 * 			Calendar object containing the date and time information to be set.
	 */
	public abstract void setDateTimeAsync(final Calendar cal,
			final XPortCallbackIF callback);

}