package de.ctnetzapplet.controller;

import java.awt.Component;
import java.awt.Container;

import javax.swing.AbstractButton;
import javax.swing.JButton;

public class SwingTools {
	private SwingTools() {}
	
	/**
	 * finds a named subcomponent within a container
	 * @param type
	 * 			Class which the component should be an instance of
	 * @param name
	 * 			name of the component to find
	 * @param container
	 * 			container to search in
	 * @param maxdepth
	 * 			maximum search-depth
	 * @return
	 * 			component or null if nothing found
	 */
	public static <T> T findComponent(Class<T> type, String name, Container container, int maxdepth) {
		T found = null;
		for(Component comp : container.getComponents()) {
			if (nameEquals(comp, name) && comp.getClass().equals(type)) {
				found = (T) comp;
			}
			else if (comp instanceof Container && maxdepth > 0) {
				found = findComponent(type, name, (Container) comp, maxdepth - 1);
			}
			if (found != null) return found;
		}
		return null;
	}
	
	/**
	 * finds a named button / button with text within a container
	 * @param name
	 * 			name of the component to find
	 * @param container
	 * 			container to search in
	 * @param maxdepth
	 * 			maximum search-depth
	 * @return
	 * 			component or null if nothing found
	 */
	public static JButton findButton(String name, Container container, int maxdepth) {
		JButton found = null;
		for(Component comp : container.getComponents()) {
			if (comp instanceof JButton) {
				JButton button = (JButton) comp;
				if (nameEquals(comp, name)) {
					found = button;
				}
				else if (textEquals(button, name)) {
					found = button;
				}
			}
			else if (comp instanceof Container && maxdepth > 0) {
				found = findButton(name, (Container) comp, maxdepth - 1);
			}
			if (found != null) return found;
		}
		return null;
	}
	
	private static boolean nameEquals(Component comp, String name) {
		String actual = comp.getName() == null ? "" : comp.getName();
		return actual.equals(name);
	}
	
	private static boolean textEquals(AbstractButton button, String text) {
		String actual = button.getText() == null ? "" : button.getText();
		return actual.equals(text);
	}

}
